//############################################################################
//
// File Name	: 'ad8555.c'
// Title		: AD8555 - Digitally Programmable Sensor Signal Zero-Drift 
//                         Amplifier
// Author		: Zoltan Gradwohl
// Created		: 09/08/2006
// Revised		: 09/08/2006
// Version		: 1.0
// Target MCU	: Atmel AVR series
// Editor Tabs	: 4
//
//	Funktionen zur Ansteuerung des AD8555 digital programmierbagen 
//  Brueckensensor Instrumentenverstaerkers 
//
// This code is distributed under the GNU Public License
//		which can be found at http://www.gnu.org/licenses/gpl.txt
//
//  ---Timing optimized for 11.0692 Mhz---
//
//############################################################################

#include <avr/io.h>
#include <avr/interrupt.h>
#include "ad8555.h"
#include "timer.h"





//############################################################################
//! enable and initialize 
void ad8555_Init(void)
//############################################################################
{
	// initialize output pin
	DDR(PORT_AD8555) |=  (1<<PIN_AD8555);		// set pin as output
	OUT(PORT_AD8555) &= ~(1<<PIN_AD8555);	    // set pin to 0

}



//############################################################################
//! write data to device 
void ad8555_write(uint8_t ad_fun, uint8_t ad_par, uint8_t ad_val)
//############################################################################
{
	// Field 0 should be written first and Field 5 written last.
	// Within each field, the MSB must be written first and the
	// LSB written last.
	
	
	uint64_t serialword 			 = (uint64_t)(AD8555_END_OF_PACKET&0x000FFF);  
	switch(ad_fun & 0x03)
	{
	// (Second Stage Gain Code): 3 LSBs Used
	case P_SECOND_STAGE_GAIN_CODE:
		serialword 	  	|= (uint64_t)(ad_val & MASK_SECOND_STAGE_GAIN_CODE)<<12;
	break;
	// (First Stage Gain Code): 7 LSBs Used
	case P_FIRST_STAGE_GAIN_CODE:
		serialword 	  	|= (uint64_t)(ad_val & MASK_FIRST_STAGE_GAIN_CODE)<<12;
	break;
	// (Output Offset Code): All 8 Bits Used
	case P_OUTPUT_OFFSET_CODE:
		serialword 	  	|= (uint64_t)(ad_val & MASK_OUTPUT_OFFSET_CODE)<<12;
	break;
	// (Other Functions)
	case P_OTHER_FUNCTIONS:
		serialword	  	|= (uint64_t)(ad_val & MASK_OTHER_FUNCTIONS)<<12;
	break;

	}
	serialword	  	   	|= (uint64_t)AD8555_DUMMY<<20;  		// dummy	
	serialword 	   		|= (uint64_t)(ad_par & 0x03)<<22;		// parameter
	serialword 	   		|= (uint64_t)(ad_fun & 0x03)<<24;		// function
	serialword 			|= (uint64_t)AD8555_START_OF_PACKET<<26;// start
	
	// reverse bits
	serialword = mirror64bits(NSERIAL_WORD_BITS, serialword);
	


	// send nbits of data to device
	send_onewirebus(NSERIAL_WORD_BITS, serialword );


}



//############################################################################
//! write data to device 
void send_onewirebus(uint8_t nbits, uint64_t data)
//############################################################################
{
	uint8_t i;
	uint8_t temp_time[3];
	// save timer settings
	temp_time[0] = TIMSK;
	temp_time[1] = TCCR1A;
	temp_time[2] = TCCR1B;
	
	// setup timer1 for delay loop
	TIMSK = 0x00; TCCR1A = 0x00;  TCCR1B = 0x00;
	timer1SetPrescaler( TIMERRTC_CLK_DIV1 );	// set prescaler
	TCNT1 = 0;									// reset TCNT1

	// clear global interrupt flag
	cli();
	// send nbits of data
	for(i=nbits;i>0;i--)
	{	
		OUT(PORT_AD8555) |= (1<<PIN_AD8555);	// set pin to 1
		if(data&0x00000001) 
		  DELAY( DELAY_US(AD8555_TW1-2) );			
		else 	      
		  DELAY( DELAY_US(AD8555_TW0-2) );			
		OUT(PORT_AD8555) &= ~(1<<PIN_AD8555);	// set pin to 0
		data>>=1;								// Tws (quasi delay)
		
		// Tw0 = 3.15 us  (at F_CPU = 11.07Mhz)
		// Tw1 = 55.2 us  (at F_CPU = 11.07Mhz)
		// Tws = 45.6 us  (at F_CPU = 11.07Mhz)
	}

	// restore timer settings
	TIMSK = temp_time[0];
	TCCR1A = temp_time[1];
	TCCR1B = temp_time[2];

	sei();

}



//############################################################################
uint64_t mirror64bits(uint8_t nbits, uint64_t in) 
//############################################################################
{
	uint64_t rev=0;
	uint8_t i;

	for (i = 0; i < nbits; i++) {
		rev = (rev << 1) | (in & 1);
		in >>=1;
	}
	return rev;
}


